==> ultocr-0.2.1.tar.gz: ultocr-0.2.1/ultocr/utils/det_utils.py <==
import numpy as np
import cv2
import torch
import torchvision.transforms as transforms


def order_points(pts):
    rect = np.zeros((4, 2), dtype="float32")
    s = pts.sum(axis=1)
    rect[0] = pts[np.argmin(s)]
    rect[2] = pts[np.argmax(s)]
    diff = np.diff(pts, axis=1)
    rect[1] = pts[np.argmin(diff)]
    rect[3] = pts[np.argmax(diff)]
    return rect


def four_point_transform(image, pts):
    rect = order_points(pts)
    (tl, tr, br, bl) = rect
    widthA = np.sqrt(((br[0] - bl[0]) ** 2) + ((br[1] - bl[1]) ** 2))
    widthB = np.sqrt(((tr[0] - tl[0]) ** 2) + ((tr[1] - tl[1]) ** 2))
    maxWidth = max(int(widthA), int(widthB))
    heightA = np.sqrt(((tr[0] - br[0]) ** 2) + ((tr[1] - br[1]) ** 2))
    heightB = np.sqrt(((tl[0] - bl[0]) ** 2) + ((tl[1] - bl[1]) ** 2))
    maxHeight = max(int(heightA), int(heightB))
    dst = np.array([
        [0, 0],
        [maxWidth - 1, 0],
        [maxWidth - 1, maxHeight - 1],
        [0, maxHeight - 1]], dtype = "float32")
    # compute the perspective transform matrix and then apply it
    M = cv2.getPerspectiveTransform(rect, dst)
    warped = cv2.warpPerspective(image, M, (maxWidth, maxHeight))
    # return the warped image
    return warped


def sort_by_line(box_info, image):
    h, w = image.shape[:2]
    scale = max(h/800, 1)
    # box_info.sort(key=lambda x: compute_center(x)[1])
    all_same_row = []
    same_row = []
    for i in range(len(box_info)-1):
        # if compute_center(box_info[i+1])[1] - compute_center(box_info[i])[1] < 5 * scale:
        #     same_row.append(box_info[i])
        if box_info[i+1][0][1] - box_info[i][0][1] < 5:
            same_row.append(box_info[i])
        else:
            same_row.append(box_info[i])
            all_same_row.append(same_row)
            same_row = []
    same_row.append(box_info[-1])
    all_same_row.append(same_row)
    sort_same_row = []
    for same in all_same_row:
        same.sort(key=lambda x: x[1][0])
        sort_same_row.append(same)
    # print('len same row:', len(sort_same_row))
    return sort_same_row


def test_preprocess(img,
                    new_size=736,
                    pad=False):
    img = test_resize(img, size=new_size, pad=pad)
    img = transforms.ToTensor()(img)
    img = transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225])(img)
    img = img.unsqueeze(0)
    return img


def test_resize(img, size=736, pad=False):
    h, w, c = img.shape
    scale_w = size / w
    scale_h = size / h
    scale = min(scale_w, scale_h)
    h = int(h * scale)
    w = int(w * scale)

    new_img = None
    if pad:
        new_img = np.zeros((size, size, c), img.dtype)
        new_img[:h, :w] = cv2.resize(img, (w, h))
    else:
        new_img = cv2.resize(img, (w, h))

    return new_img


def draw_bbox(img, result, color=(255, 0, 0), thickness=2):
    """
    :input: RGB img
    """
    if isinstance(img, str):
        img = cv2.imread(img)
    img = img.copy()
    h, w = img.shape[:2]
    for point in result:
        # point = point.astype(int)
        point = np.array(point).astype(int)
        point = point.reshape((-1, 1, 2))  
        cv2.polylines(img, [point], True, color, thickness)
    return img

==> ultocr-0.3.1.tar.gz: ultocr-0.3.1/ultocr/utils/det_utils.py <==
import numpy as np
import cv2
import torch
import torchvision.transforms as transforms


def order_points(pts):
    rect = np.zeros((4, 2), dtype="float32")
    s = pts.sum(axis=1)
    rect[0] = pts[np.argmin(s)]
    rect[2] = pts[np.argmax(s)]
    diff = np.diff(pts, axis=1)
    rect[1] = pts[np.argmin(diff)]
    rect[3] = pts[np.argmax(diff)]
    return rect


def four_point_transform(image, pts):
    rect = order_points(pts)
    (tl, tr, br, bl) = rect
    widthA = np.sqrt(((br[0] - bl[0]) ** 2) + ((br[1] - bl[1]) ** 2))
    widthB = np.sqrt(((tr[0] - tl[0]) ** 2) + ((tr[1] - tl[1]) ** 2))
    maxWidth = max(int(widthA), int(widthB))
    heightA = np.sqrt(((tr[0] - br[0]) ** 2) + ((tr[1] - br[1]) ** 2))
    heightB = np.sqrt(((tl[0] - bl[0]) ** 2) + ((tl[1] - bl[1]) ** 2))
    maxHeight = max(int(heightA), int(heightB))
    dst = np.array([
        [0, 0],
        [maxWidth - 1, 0],
        [maxWidth - 1, maxHeight - 1],
        [0, maxHeight - 1]], dtype = "float32")
    # compute the perspective transform matrix and then apply it
    M = cv2.getPerspectiveTransform(rect, dst)
    warped = cv2.warpPerspective(image, M, (maxWidth, maxHeight))
    # return the warped image
    return warped


def is_on_same_line(box_a, box_b, min_y_overlap_ratio=0.8):
    """Check if two boxes are on the same line by their y-axis coordinates.

    Two boxes are on the same line if they overlap vertically, and the length
    of the overlapping line segment is greater than min_y_overlap_ratio * the
    height of either of the boxes.

    Args:
        box_a (list), box_b (list): Two bounding boxes to be checked
        min_y_overlap_ratio (float): The minimum vertical overlapping ratio
                                    allowed for boxes in the same line

    Returns:
        The bool flag indicating if they are on the same line
    """
    a_y_min = np.min(box_a[1::2])
    b_y_min = np.min(box_b[1::2])
    a_y_max = np.max(box_a[1::2])
    b_y_max = np.max(box_b[1::2])

    # Make sure that box a is always the box above another
    if a_y_min > b_y_min:
        a_y_min, b_y_min = b_y_min, a_y_min
        a_y_max, b_y_max = b_y_max, a_y_max

    if b_y_min <= a_y_max:
        if min_y_overlap_ratio is not None:
            sorted_y = sorted([b_y_min, b_y_max, a_y_max])
            overlap = sorted_y[1] - sorted_y[0]
            min_a_overlap = (a_y_max - a_y_min) * min_y_overlap_ratio
            min_b_overlap = (b_y_max - b_y_min) * min_y_overlap_ratio
            return overlap >= min_a_overlap or \
                overlap >= min_b_overlap
        else:
            return True
    return False


def sort_by_line(box_info, image):
    h, w = image.shape[:2]
    scale = max(h/800, 1)
    # box_info.sort(key=lambda x: compute_center(x)[1])
    all_same_row = []
    same_row = []
    for i in range(len(box_info)-1):
        if is_on_same_line(np.array(box_info[i+1]).reshape(-1).tolist(), np.array(box_info[i]).reshape(-1).tolist()):
            same_row.append(box_info[i])
        # if compute_center(box_info[i+1])[1] - compute_center(box_info[i])[1] < 5 * scale:
        #     same_row.append(box_info[i])
        # if box_info[i+1][0][1] - box_info[i][0][1] < 5:
        #     same_row.append(box_info[i])
        else:
            same_row.append(box_info[i])
            all_same_row.append(same_row)
            same_row = []
    same_row.append(box_info[-1])
    all_same_row.append(same_row)
    sort_same_row = []
    for same in all_same_row:
        same.sort(key=lambda x: x[1][0])
        sort_same_row.append(same)
    # print('len same row:', len(sort_same_row))
    return sort_same_row


def test_preprocess(img,
                    new_size=736,
                    pad=False):
    img = test_resize(img, size=new_size, pad=pad)
    img = transforms.ToTensor()(img)
    img = transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225])(img)
    img = img.unsqueeze(0)
    return img


def test_resize(img, size=736, pad=False):
    h, w, c = img.shape
    scale_w = size / w
    scale_h = size / h
    scale = min(scale_w, scale_h)
    h = int(h * scale)
    w = int(w * scale)

    new_img = None
    if pad:
        new_img = np.zeros((size, size, c), img.dtype)
        new_img[:h, :w] = cv2.resize(img, (w, h))
    else:
        new_img = cv2.resize(img, (w, h))

    return new_img


def draw_bbox(img, result, color=(255, 0, 0), thickness=2):
    """
    :input: RGB img
    """
    if isinstance(img, str):
        img = cv2.imread(img)
    img = img.copy()
    h, w = img.shape[:2]
    for point in result:
        # point = point.astype(int)
        point = np.array(point).astype(int)
        point = point.reshape((-1, 1, 2))  
        cv2.polylines(img, [point], True, color, thickness)
    return img
